"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createParamDecorator = exports.bindControllerRoutes = void 0;
require("reflect-metadata");
const controller_meta_1 = require("../metadata/controller.meta");
const zod_1 = require("zod");
const tsyringe_1 = require("tsyringe");
function bindControllerRoutes(api, controllers) {
    const routes = {};
    controllers.forEach((controller) => {
        const ctrl = controller_meta_1.controllersMeta.get(controller.prototype);
        if (!ctrl || typeof ctrl.ref !== 'function') {
            throw new Error(`${controller.name} missing @Controller decorator`);
        }
        const classInstance = tsyringe_1.container.resolve(ctrl.ref);
        if (!classInstance) {
            throw new Error(`${controller.name} missing @inject decorator`);
        }
        ctrl.methods.forEach(method => {
            let route = routes[ctrl.name];
            if (!route) {
                routes[ctrl.name] = route = [];
            }
            route.push(method.bindToApi(api, ctrl.path, classInstance));
        });
    });
    return routes;
}
exports.bindControllerRoutes = bindControllerRoutes;
function createParamDecorator(getter) {
    function ParamConstructor(p1, p2) {
        let data = undefined;
        let validator = undefined;
        if (p1 instanceof zod_1.ZodObject) {
            validator = p1;
        }
        else if (p1) {
            data = p1;
            if (p2 instanceof zod_1.ZodObject) {
                validator = p2;
            }
        }
        return (proto, name, index) => {
            new controller_meta_1.ControllerMeta(proto)
                .getMethod(name)
                .params[index] = { getter, validator, data };
        };
    }
    return ParamConstructor;
}
exports.createParamDecorator = createParamDecorator;
