"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MethodMeta = void 0;
const http_errors_1 = __importDefault(require("http-errors"));
const path_1 = require("path");
class MethodMeta {
    name = '';
    path = '';
    action = '';
    ref = null;
    guards = [];
    params = [];
    constructor(name) {
        this.name = name;
    }
    bindToApi(api, basePath, actionClass) {
        const actionMethod = this.ref;
        if (actionMethod == null) {
            throw new Error(`${this.name} method missing route decorator`);
        }
        const routePath = this.toRoutePath(basePath);
        const routeAction = this.toRouteAction(actionMethod, actionClass);
        api[this.action](routePath, routeAction);
        return {
            action: this.action.toUpperCase(),
            path: routePath,
            method: this.name,
            guards: this.guards
        };
    }
    toRoutePath(basePath) {
        return '/' +
            (0, path_1.join)(basePath, this.path)
                .replace(/^\//, '');
    }
    toRouteAction(actionMethod, actionClass) {
        const routeParams = this.toRouteParams(actionClass);
        const routeGuard = this.toRouteGuard();
        return async function routeAction(req, res) {
            await routeGuard(req);
            const actionParams = routeParams.map(param => {
                let value;
                if (param.getter) {
                    value = param.getter(param.data, req, res);
                }
                if (param.validator) {
                    value = param.validator.parse(value);
                }
                else if (typeof param.type === 'function') {
                    value = param.type(value);
                }
                return value;
            });
            return actionMethod.apply(actionClass, actionParams);
        };
    }
    toRouteParams(actionClass) {
        const types = Reflect.getMetadata("design:paramtypes", actionClass, this.name);
        return types.map((type, i) => {
            const decoratorParams = this.params[i];
            if (!decoratorParams) {
                throw new Error(`${this.name} method missing param decorator at index ${i}`);
            }
            return { ...decoratorParams, type };
        });
    }
    toRouteGuard() {
        const guards = this.guards;
        const guardLen = guards.length;
        if (!guardLen) {
            throw new Error(`${this.name} method missing a guard`);
        }
        return async function routeGuard(req) {
            for (let i = 0; i < guardLen; ++i) {
                const result = await guards[i](req);
                if (result)
                    return;
            }
            throw new http_errors_1.default.Unauthorized();
        };
    }
}
exports.MethodMeta = MethodMeta;
