"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setResponse = exports.getBody = exports.getHeader = exports.getHeaders = exports.getCookies = exports.getQuery = exports.getIp = void 0;
const querystring_1 = require("querystring");
const utils_1 = require("./utils");
function getIp(req) {
    return 'not-implemented';
}
exports.getIp = getIp;
function getQuery(req) {
    const [rawPath, rawQueryString] = req.url.split('?');
    const queryParams = Object.entries((0, querystring_1.parse)(rawQueryString))
        .reduce((out, [key, val]) => {
        out[key] = Array.isArray(val)
            ? val.join(',')
            : val;
        return out;
    }, {});
    return { rawPath, rawQueryString, queryParams };
}
exports.getQuery = getQuery;
function getCookies(req) {
    const cookies = getHeader(req, 'Cookie');
    if (!cookies) {
        return [];
    }
    return Array.isArray(cookies)
        ? cookies
        : cookies.split(';').map(s => s.trim());
}
exports.getCookies = getCookies;
function getHeaders(req) {
    return Object.entries(req.headers).reduce((out, [key, val]) => {
        out[key] = Array.isArray(val)
            ? val.join(',')
            : val;
        return out;
    }, {});
}
exports.getHeaders = getHeaders;
function getHeader(req, key) {
    const value = req.headers[key]
        || req.headers[key.toLowerCase()]
        || '';
    return Array.isArray(value)
        ? value.join(',')
        : value;
}
exports.getHeader = getHeader;
async function getBody(req) {
    if (req.method === 'GET'
        || req.method === 'HEAD') {
        return undefined;
    }
    try {
        const buffers = [];
        for await (const chunk of req) {
            buffers.push(chunk);
        }
        return Buffer
            .concat(buffers)
            .toString('utf8');
    }
    catch (err) {
        return undefined;
    }
}
exports.getBody = getBody;
function setResponse(res, result) {
    if ((0, utils_1.isError)(result)) {
        res.writeHead(result.statusCode || result.code || 500, { 'Content-Type': 'text/plain' });
        res.write((0, utils_1.toString)(result));
    }
    else if (!(0, utils_1.isEvent)(result)) {
        res.writeHead(result.statusCode || result.code || 200, { 'Content-Type': 'application/json' });
        res.write((0, utils_1.toString)(result));
    }
    else {
        res.statusCode = result.statusCode;
        res.writeHead(result.statusCode, (0, utils_1.toObject)(result.headers));
        if ((0, utils_1.isPlainObject)(result.multiValueHeaders)) {
            Object.entries(result.multiValueHeaders).forEach(([key, values]) => {
                Array.isArray(values)
                    ? values.forEach(value => res.setHeader(key, value))
                    : res.setHeader(key, values);
            });
        }
        res.write((0, utils_1.toString)(result.body), result.isBase64Encoded
            ? 'base64'
            : 'utf8');
    }
}
exports.setResponse = setResponse;
